@@ -1,6 +1,16 @@
+import re
+
+try:
+    from bson import (ObjectId, RE_TYPE)
+except ImportError:
+    from mongomock.object_id import ObjectId
+    RE_TYPE = type(re.compile(''))
+
+
 #for Python 3 compatibility
 try:
   unicode = unicode
+  from __builtin__ import basestring
 except NameError:
   unicode = str
   basestring = (str, bytes)
