@@ -4,13 +4,12 @@ import itertools
 import re
 import platform
 import sys
-if platform.python_version() < '2.7':
-    import unittest2 as unittest
-else:
-    import unittest
+
+from .utils import TestCase, skipIf
 
 import mongomock
 from mongomock import Database
+
 try:
     import pymongo
     from pymongo import Connection as PymongoConnection
@@ -30,10 +29,6 @@ except ImportError:
 from tests.multicollection import MultiCollection
 
 
-
-class TestCase(unittest.TestCase):
-    pass
-
 class InterfaceTest(TestCase):
     def test__can_create_db_without_path(self):
         conn = mongomock.Connection()
@@ -61,52 +56,8 @@ class DatabaseGettingTest(TestCase):
         self.assertIs(db1, self.conn.some_database_here)
         self.assertIsInstance(db1, Database)
 
-class CollectionAPITest(TestCase):
-    def setUp(self):
-        super(CollectionAPITest, self).setUp()
-        self.conn = mongomock.Connection()
-        self.db = self.conn['somedb']
-    def test__get_collection_names(self):
-        self.db.a
-        self.db.b
-        self.assertEquals(set(self.db.collection_names()), set(['a', 'b', 'system.indexes']))
-    def test__drop_collection(self):
-        self.db.a
-        self.db.b
-        self.db.c
-        self.db.drop_collection('b')
-        self.db.drop_collection('b')
-        self.db.drop_collection(self.db.c)
-        self.assertEquals(set(self.db.collection_names()), set(['a', 'system.indexes']))
 
-    def test__getting_collection_via_getattr(self):
-        col1 = self.db.some_collection_here
-        col2 = self.db.some_collection_here
-        self.assertIs(col1, col2)
-        self.assertIs(col1, self.db['some_collection_here'])
-        self.assertIsInstance(col1, mongomock.Collection)
-
-    def test__getting_collection_via_getitem(self):
-        col1 = self.db['some_collection_here']
-        col2 = self.db['some_collection_here']
-        self.assertIs(col1, col2)
-        self.assertIs(col1, self.db.some_collection_here)
-        self.assertIsInstance(col1, mongomock.Collection)
-
-    def test__find_returns_cursors(self):
-        collection = self.db.collection
-        self.assertEquals(type(collection.find()).__name__, "Cursor")
-        self.assertNotIsInstance(collection.find(), list)
-        self.assertNotIsInstance(collection.find(), tuple)
-
-    def test__naming_bug(self):
-        self.db['abc'].save({'name':'test1'})
-        self.db['abc'].save({'name':'test2'})
-        #now searching for 'name':'e' returns test1
-        self.assertIsNone(self.db['abc'].find_one({'name':'e'}))
-
-
-@unittest.skipIf(not _HAVE_PYMONGO,"pymongo not installed")
+@skipIf(not _HAVE_PYMONGO,"pymongo not installed")
 class _CollectionComparisonTest(TestCase):
     """Compares a fake collection with the real mongo collection implementation via cross-comparison."""
 
@@ -122,7 +73,7 @@ class _CollectionComparisonTest(TestCase):
             "real": self.mongo_conn[self.db_name][self.collection_name],
          })
 
-    def _connect_to_local_mongodb(self, num_retries=3):
+    def _connect_to_local_mongodb(self, num_retries=60):
         "Performs retries on connection refused errors (for travis-ci builds)"
         connection_class = self._get_real_connection_class()
         for retry in range(num_retries):
@@ -185,6 +136,11 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.compare.find_one({"_id" : "id1"})
         self.cmp.do.insert({"_id":"id2", "name" : "another new"})
         self.cmp.compare.find_one({"_id" : "id2"}, {"_id":1})
+        self.cmp.compare.find_one("id2", {"_id":1})
+
+    def test__find_one_no_args(self):
+        self.cmp.do.insert({"_id": "new_obj", "field": "value"})
+        self.cmp.compare.find_one()
 
     def test__find_by_attributes(self):
         id1 = ObjectId()
@@ -255,6 +211,24 @@ class _CollectionTest(_CollectionComparisonTest):
         regex = re.compile('bob|notsam')
         self.cmp.compare_ignore_order.find({"name" : regex})
 
+    def test__find_by_elemMatch(self):
+        self.cmp.do.insert({"field": [{"a": 1, "b": 2}, {"c": 3, "d": 4}]})
+        self.cmp.do.insert({"field": [{"a": 1, "b": 4}, {"c": 3, "d": 8}]})
+        self.cmp.do.insert({"field": "nonlist"})
+        self.cmp.do.insert({"field": 2})
+
+        self.cmp.compare.find({"field": {"$elemMatch": {"b": 1}}})
+        self.cmp.compare_ignore_order.find({"field": {"$elemMatch": {"a": 1}}})
+        self.cmp.compare.find({"field": {"$elemMatch": {"b": {"$gt": 3}}}})
+
+    def test__find_in_array(self):
+        self.cmp.do.insert({"field": [{"a": 1, "b": 2}, {"c": 3, "d": 4}]})
+
+        self.cmp.compare.find({"field.0.a": 1})
+        self.cmp.compare.find({"field.0.b": 2})
+        self.cmp.compare.find({"field.1.c": 3})
+        self.cmp.compare.find({"field.1.d": 4})
+
     def test__find_notequal(self):
         """Test searching with operators other than equality."""
         bob = {'_id': 1, 'name': 'bob'}
@@ -304,6 +278,11 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.compare_ignore_order.find({'$or':[{'x':4}, {'x':7}]})
         self.cmp.compare_ignore_order.find({'$and':[{'x':2}, {'x':7}]})
 
+    def test__find_and_modify_remove(self):
+        self.cmp.do.insert([{"a": x} for x in range(10)])
+        self.cmp.do.find_and_modify({"a": 2}, remove=True)
+        self.cmp.compare_ignore_order.find()
+
     def test__find_sort_list(self):
         self.cmp.do.remove()
         for data in ({"a" : 1, "b" : 3, "c" : "data1"},
@@ -321,6 +300,15 @@ class _CollectionTest(_CollectionComparisonTest):
                      {"a" : 3, "b" : 1, "c" : "data2"}):
             self.cmp.do.insert(data)
         self.cmp.compare.find(limit=2, sort = [("a", 1), ("b", -1)])
+        self.cmp.compare.find(limit=0, sort = [("a", 1), ("b", -1)]) #pymongo limit defaults to 0, returning everything
+
+    def test__as_class(self):
+        class MyDict(dict): pass
+
+        self.cmp.do.remove()
+        self.cmp.do.insert({"a": 1, "b": {"ba": 3, "bb": 4, "bc": [ {"bca": 5 } ] }})
+        self.cmp.compare.find({}, as_class=MyDict)
+        self.cmp.compare.find({"a": 1}, as_class=MyDict)
 
     def test__return_only_selected_fields(self):
         self.cmp.do.insert({'name':'Chucky', 'type':'doll', 'model':'v6'})
@@ -365,6 +353,33 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.do.update({'name': 'bob'}, {'$set': {'hat': 'red'}})
         self.cmp.compare.find({'name': 'bob'})
 
+    def test__unset(self):
+        """Tests calling update with $set members."""
+        self.cmp.do.update({'name': 'bob'}, {'a': 'aaa'}, upsert=True)
+        self.cmp.compare.find({'name' : 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$unset': {'a': 0}})
+        self.cmp.compare.find({'name' : 'bob'})
+
+        self.cmp.do.update({'name': 'bob'}, {'a': 'aaa'}, upsert=True)
+        self.cmp.compare.find({'name' : 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$unset': {'a': 1}})
+        self.cmp.compare.find({'name' : 'bob'})
+
+        self.cmp.do.update({'name': 'bob'}, {'a': 'aaa'}, upsert=True)
+        self.cmp.compare.find({'name' : 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$unset': {'a': ""}})
+        self.cmp.compare.find({'name' : 'bob'})
+
+        self.cmp.do.update({'name': 'bob'}, {'a': 'aaa'}, upsert=True)
+        self.cmp.compare.find({'name' : 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$unset': {'a': True}})
+        self.cmp.compare.find({'name' : 'bob'})
+
+        self.cmp.do.update({'name': 'bob'}, {'a': 'aaa'}, upsert=True)
+        self.cmp.compare.find({'name' : 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$unset': {'a': False}})
+        self.cmp.compare.find({'name' : 'bob'})
+
     def test__set_upsert(self):
         self.cmp.do.remove()
         self.cmp.do.update({"name": "bob"}, {"$set": {}}, True)
@@ -405,6 +420,13 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.do.update({'name':'bob'}, {'$inc': {'data.age2': 1}})
         self.cmp.compare.find()
 
+    def test__inc_subdocument_positional(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'data': [{'age': 0}, {'age': 1}]})
+        self.cmp.do.update({'name': 'bob', 'data': {'$elemMatch': {'age': 0}}},
+            {'$inc': {'data.$.age': 1}})
+        self.cmp.compare.find()
+
     def test__addToSet(self):
         self.cmp.do.remove()
         self.cmp.do.insert({'name': 'bob'})
@@ -412,13 +434,61 @@ class _CollectionTest(_CollectionComparisonTest):
             self.cmp.do.update({'name':'bob'}, {'$addToSet': {'hat':'green'}})
             self.cmp.compare.find({'name': 'bob'})
         for i in range(3):
-            self.cmp.do.update({'name':'bob'}, {'$addToSet': {'hat':'tall'}})
+            self.cmp.do.update({'name': 'bob'}, {'$addToSet': {'hat':'tall'}})
             self.cmp.compare.find({'name': 'bob'})
 
     def test__pull(self):
         self.cmp.do.remove()
-        self.cmp.do.insert({'name': 'bob', 'hat':['green', 'tall']})
-        self.cmp.do.update({'name':'bob'}, {'$pull': {'hat':'green'}})
+        self.cmp.do.insert({'name': 'bob', 'hat': ['green', 'tall']})
+        self.cmp.do.update({'name': 'bob'}, {'$pull': {'hat': 'green'}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__pull_nested_dict(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': [{'name': 'derby', 'sizes': [{'size': 'L', 'quantity': 3}, {'size': 'XL', 'quantity': 4}], 'colors': ['green', 'blue']}, {'name': 'cap', 'sizes': [{'size': 'S', 'quantity': 10}, {'size': 'L', 'quantity': 5}], 'colors': ['blue']}]})
+        self.cmp.do.update({'hat': {'$elemMatch': {'name': 'derby'}}}, {'$pull': {'hat.$.sizes': {'size': 'L'}}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__pull_nested_list(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': [{'name': 'derby', 'sizes': ['L', 'XL']}, {'name': 'cap', 'sizes': ['S', 'L']}]})
+        self.cmp.do.update({'hat': {'$elemMatch': {'name': 'derby'}}}, {'$pull': {'hat.$.sizes': 'XL'}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': ['green', 'tall']})
+        self.cmp.do.update({'name': 'bob'}, {'$push': {'hat': 'wide'}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push_dict(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': [{'name': 'derby', 'sizes': ['L', 'XL']}]})
+        self.cmp.do.update({'name': 'bob'}, {'$push': {'hat': {'name': 'cap', 'sizes': ['S', 'L']}}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push_each(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': ['green', 'tall']})
+        self.cmp.do.update({'name': 'bob'}, {'$push': {'hat': {'$each': ['wide', 'blue']}}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push_nested_dict(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': [{'name': 'derby', 'sizes': [{'size': 'L', 'quantity': 3}, {'size': 'XL', 'quantity': 4}], 'colors': ['green', 'blue']}, {'name': 'cap', 'sizes': [{'size': 'S', 'quantity': 10}, {'size': 'L', 'quantity': 5}], 'colors': ['blue']}]})
+        self.cmp.do.update({'hat': {'$elemMatch': {'name': 'derby'}}}, {'$push': {'hat.$.sizes': {'size': 'M', 'quantity': 6}}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push_nested_dict_each(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': [{'name': 'derby', 'sizes': [{'size': 'L', 'quantity': 3}, {'size': 'XL', 'quantity': 4}], 'colors': ['green', 'blue']}, {'name': 'cap', 'sizes': [{'size': 'S', 'quantity': 10}, {'size': 'L', 'quantity': 5}], 'colors': ['blue']}]})
+        self.cmp.do.update({'hat': {'$elemMatch': {'name': 'derby'}}}, {'$push': {'hat.$.sizes': {'$each': [{'size': 'M', 'quantity': 6}, {'size': 'S', 'quantity': 1}]}}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push_nested_list_each(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob', 'hat': [{'name': 'derby', 'sizes': ['L', 'XL'], 'colors': ['green', 'blue']}, {'name': 'cap', 'sizes': ['S', 'L'], 'colors': ['blue']}]})
+        self.cmp.do.update({'hat': {'$elemMatch': {'name': 'derby'}}}, {'$push': {'hat.$.sizes': {'$each': ['M', 'S']}}})
         self.cmp.compare.find({'name': 'bob'})
 
     def test__drop(self):
@@ -439,8 +509,8 @@ class MongoClientCollectionTest(_CollectionTest, _MongoClientMixin):
 class PymongoCollectionTest(_CollectionTest, _PymongoConnectionMixin):
     pass
 
-@unittest.skipIf(not _HAVE_PYMONGO,"pymongo not installed")
-@unittest.skipIf(not _HAVE_MAP_REDUCE,"execjs not installed")
+@skipIf(not _HAVE_PYMONGO,"pymongo not installed")
+@skipIf(not _HAVE_MAP_REDUCE,"execjs not installed")
 class CollectionMapReduceTest(TestCase):
     def setUp(self):
         self.db = mongomock.Connection().map_reduce_test
